package org.processmining.models;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.util.HTMLToString;
import org.processmining.framework.util.Pair;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.causalactivitygraph.models.CausalActivityGraph instead.
 */
@Deprecated
public interface CausalActivityGraph extends HTMLToString {

	/**
	 * Initializes the graph with the given set of activities. All activities
	 * will be set, all causalities will be reset.
	 * 
	 * @param label
	 *            The label to use for this graph.
	 * @param activities
	 *            The given set of activities.
	 */
	void init(String label, Set<XEventClass> activities);

	/**
	 * Returns the label for this graph.
	 * 
	 * @return The label for this graph.
	 */
	String getLabel();
	
	/**
	 * Sets the given activity.
	 * 
	 * @param activity
	 *            The given activity.
	 */
	void setActivity(XEventClass activity);

	/**
	 * Sets the causality from the given first activity to the given second
	 * activity. Sets both activities if they are not yet set.
	 * 
	 * @param fromActivity
	 *            The given first activity.
	 * @param toActivity
	 *            The given second activity.
	 */
	void setCausality(XEventClass fromActivity, XEventClass toActivity);

	/**
	 * Sets the causality from the given first activity to the given second
	 * activity to the given weight. Sets both activities if they are not yet set.
	 * 
	 * @param fromActivity
	 *            The given first activity.
	 * @param toActivity
	 *            The given second activity.
	 * @param weight
	 *            The given weight.
	 */
	void setCausality(XEventClass fromActivity, XEventClass toActivity, double weight);

	/**
	 * Resets the given activity. Resets all causalities connected to this
	 * activity.
	 * 
	 * @param activity
	 *            The given activity.
	 */
	void resetActivity(XEventClass activity);

	/**
	 * Resets the causality from the given first activity to the given second
	 * activity.
	 * 
	 * @param fromActivity
	 *            The given first activity.
	 * @param toActivity
	 *            The given second activity.
	 */
	void resetCausality(XEventClass fromActivity, XEventClass toActivity);

	/**
	 * Returns the (ordered) list of activities in the graph. This includes
	 * activities that have not been set.
	 * 
	 * @return The (ordered) list of activities in the graph.
	 */
	List<XEventClass> getActivities();

	/**
	 * Returns the set of set activities in the graph.
	 * 
	 * @return The set of set activities in the graph.
	 */
	Set<XEventClass> getSetActivities();

	/**
	 * Returns the weight of the given edge in the graph.
	 * 
	 * @return The weight of the given edge in the graph.
	 */
	double getCausality(XEventClass fromActivity, XEventClass toActivity);
	
	/**
	 * Returns the set of set causalities in the graph.
	 * 
	 * @return The set of set causalities in the graph.
	 */
	Set<Pair<XEventClass, XEventClass>> getSetCausalities();

	/**
	 * Returns the set of causalities in the graph with weight more than the given threshold.
	 * 
	 * @param threshold
	 * 			The given threshold.
	 * 
	 * @return The set of set causalities in the graph.
	 */
	Set<Pair<XEventClass, XEventClass>> getSetCausalities(double threshold);

	/**
	 * Returns the preset of the given set activity, that is, the set of set
	 * activities that a outgoing causalities to the given activity.
	 * 
	 * @param activity
	 *            The given set activity.
	 * @return The preset of the given activity.
	 */
	Set<XEventClass> getPreSetActivities(XEventClass activity);

	/**
	 * Returns the postset of the given set activity, that is, the set of set
	 * activities that a incoming causalities from the given activity.
	 * 
	 * @param activity
	 *            The given set activity.
	 * @return The postset of the given set activity.
	 */
	Set<XEventClass> getPostSetActivities(XEventClass activity);

	/**
	 * Returns the set activities in the graph that have no incoming
	 * causalities.
	 * 
	 * @return The set activities in the graph that have no incoming
	 *         causalities.
	 */
	Set<XEventClass> getSourceSetActivities();

	/**
	 * Returns the set activities in the graph that have no outgoing
	 * causalities.
	 * 
	 * @return The set activities in the graph that have no outgoing
	 *         causalities.
	 */
	Set<XEventClass> getSinkSetActivities();
	
	/**
	 * Gets the value how good the given graph matches this graph:
	 * - 1.0 means a perfect positive match, both graphs are equal.
	 * - 0.0 means no match, like both graphs are random graphs.
	 * - -1.0 means a perfect negative match.
	 * - NaN means that there was no overlap in the graphs: No activities are present in both graphs.
	 * 
	 * @param graph
	 * @return The value as described above.
	 */
	double getMatch(CausalActivityGraph graph);
	
	public void importFromStream(InputStream input) throws IOException;
	public void exportToFile(File file) throws IOException;
}
