package org.processmining.models.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.util.Pair;
import org.processmining.models.TraceAlignment;
import org.processmining.plugins.petrinet.replayresult.StepTypes;

import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.logalignment.models.impl.TraceAlignmentImpl instead.
 */
@Deprecated
public class TraceAlignmentImpl implements TraceAlignment {

	/*
	 * We only support step types LMGOOD (sync move), L (log move), and MREAL
	 * (model move).
	 */
	private List<Pair<StepTypes, Pair<XEventClass, String>>> legalMoves;
	private double costs;

	protected TraceAlignmentImpl() {
	}

	public void init() {
		legalMoves = new ArrayList<Pair<StepTypes, Pair<XEventClass, String>>>();
		setCosts(0.0);
	}

	public List<Pair<StepTypes, Pair<XEventClass, String>>> getLegalMoves() {
		return legalMoves;
	}

	public Pair<StepTypes, Pair<XEventClass, String>> getLegalMove(int index) {
		return legalMoves.get(index);
	}

	public void setLegalMoves(List<Pair<StepTypes, Pair<XEventClass, String>>> legalMoves) {
		this.legalMoves = legalMoves;
	}

	public void addLegalMove(StepTypes stepType, XEventClass activity, String transitionId) {
		legalMoves.add(new Pair<StepTypes, Pair<XEventClass, String>>(stepType, new Pair<XEventClass, String>(activity,
				transitionId)));
	}

	public int getSize() {
		return legalMoves.size();
	}

	public List<XEventClass> getLogMoves() {
		List<XEventClass> trace = new ArrayList<XEventClass>();
		for (Pair<StepTypes, Pair<XEventClass, String>> legalMove : legalMoves) {
			if (legalMove.getFirst() == StepTypes.LMGOOD || legalMove.getFirst() == StepTypes.L) {
				trace.add(legalMove.getSecond().getFirst());
			}
		}
		return trace;
	}

	public String toHTMLString(boolean includeHTMLTags) {
		StringBuffer buffer = new StringBuffer();
		if (includeHTMLTags) {
			buffer.append("<html>");
			buffer.append("<h1>Trace alignment</h1>");
		}
		buffer.append("<table><tr>");
		for (int index = 0; index < legalMoves.size(); index++) {
			if (legalMoves.get(index).getFirst() == StepTypes.LMGOOD || legalMoves.get(index).getFirst() == StepTypes.L) {
				buffer.append("<td>" + legalMoves.get(index).getSecond().getFirst() + "</td>");
			} else {
				buffer.append("<td>&gt;&gt;</td>");
			}
		}
		buffer.append("</tr><tr>");
		for (int index = 0; index < legalMoves.size(); index++) {
			if (legalMoves.get(index).getFirst() == StepTypes.LMGOOD
					|| legalMoves.get(index).getFirst() == StepTypes.MREAL
				|| legalMoves.get(index).getFirst() == StepTypes.MINVI) {
				buffer.append("<td>" + legalMoves.get(index).getSecond().getSecond() + "</td>");
			} else {
				buffer.append("<td>&gt;&gt;</td>");
			}
		}
		buffer.append("</tr></table>");
		buffer.append("<h2>Costs</h2>" + getCosts());
		if (includeHTMLTags) {
			buffer.append("</html>");
		}
		return buffer.toString();
	}

	public void exportToCSVFile(CsvWriter writer) throws IOException {
		writer.write("COSTS");
		writer.write(Double.toString(getCosts()));
		for (Pair<StepTypes, Pair<XEventClass, String>> legalMove : legalMoves) {
			switch (legalMove.getFirst()) {
				case LMGOOD :
					writer.write("LMGOOD");
					writer.write(legalMove.getSecond().getFirst().getId());
					writer.write(legalMove.getSecond().getSecond());
					break;
				case L :
					writer.write("L");
					writer.write(legalMove.getSecond().getFirst().getId());
					writer.write("N/A");
					break;
				case MREAL :
					writer.write("MREAL");
					writer.write("N/A");
					writer.write(legalMove.getSecond().getSecond());
					break;
				case MINVI :
					writer.write("MINVI");
					writer.write("N/A");
					writer.write(legalMove.getSecond().getSecond());
					break;
				default :
					writer.write("");
					break;
			}
		}
		writer.write("END");
		writer.endRecord();
	}

	public void importFromCSVFile(CsvReader reader, Map<String, XEventClass> activities) throws IOException {
		init();
		for (int i = 0; i < reader.getColumnCount();) {
			String tag = reader.get(i++);
			if (tag.equals("LMGOOD") || tag.equals("L") || tag.equals("MREAL") || tag.equals("MINVI")) {
				if (i + 1 < reader.getColumnCount()) {
					String activityId = reader.get(i++);
					String transitionId = reader.get(i++);
					if (!activities.keySet().contains(activityId)) {
						activities.put(activityId, new XEventClass(activityId, activities.keySet().size()));
					}
					XEventClass activity = activities.get(activityId);
					if (tag.equals("LMGOOD")) {
						addLegalMove(StepTypes.LMGOOD, activity, transitionId);
					} else if (tag.equals("L")) {
						addLegalMove(StepTypes.L, activity, null);
					} else if (tag.equals("MREAL")) {
						addLegalMove(StepTypes.MREAL, null, transitionId);
					} else if (tag.equals("MINVI")) {
						addLegalMove(StepTypes.MINVI, null, transitionId);
					} else {
						// Ignore
					}
				}
			} else if (tag.equals("COSTS")) {
				if (i < reader.getColumnCount()) {
					Double costs = Double.parseDouble(reader.get(i++));
					setCosts(costs);
				}
			}
		}
	}

	public void setCosts(double costs) {
		this.costs = costs;
	}

	public double getCosts() {
		return costs;
	}
}
