package org.processmining.plugins;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.connections.PackPetrinetConnection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.AcceptingPetriNet;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.impl.DivideAndConquerFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.parameters.PackPetrinetParameters;
import org.processmining.utils.PetriNetUtils;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.acceptingpetrinet.plugins.PackPetrinetPlugin instead.
 */
@Deprecated
//Plugin(name = "Pack Petri net", parameterLabels = { "Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class })
public class PackPetrinetPlugin {

//	UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	PluginVariant(variantLabel = "Pack Accepting Petri net, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet packDefault(PluginContext context, Petrinet net) {
		PackPetrinetParameters parameters = new PackPetrinetParameters();
		return packPrivateConnection(context, net, parameters);
	}

//	PluginVariant(variantLabel = "Pack Accepting Petri net, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet packParameters(PluginContext context, Petrinet net, PackPetrinetParameters parameters) {
		return packPrivateConnection(context, net, parameters);
	}

	private AcceptingPetriNet packPrivateConnection(PluginContext context, Petrinet net,
			PackPetrinetParameters parameters) {
		Collection<PackPetrinetConnection> connections;
		try {
			connections = context.getConnectionManager().getConnections(PackPetrinetConnection.class, context, net);
			for (PackPetrinetConnection connection : connections) {
				if (connection.getObjectWithRole(PackPetrinetConnection.NET).equals(net)
						&& connection.getParameters().equals(parameters)) {
					return connection.getObjectWithRole(PackPetrinetConnection.APN);
				}
			}
		} catch (ConnectionCannotBeObtained e) {
		}
		AcceptingPetriNet apn = packPrivate(context, net, parameters);
		context.getConnectionManager().addConnection(new PackPetrinetConnection(apn, net, parameters));
		return apn;
	}

	private AcceptingPetriNet packPrivate(PluginContext context, Petrinet net, PackPetrinetParameters parameters) {
		Map<Place, Place> placeMap = new HashMap<Place, Place>(net.getPlaces().size());
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>(net.getTransitions()
				.size());
		Petrinet clonedNet = PetriNetUtils.cloneNet(net, placeMap, transitionMap);
		AcceptingPetriNet apn = DivideAndConquerFactory.createAcceptingPetriNet();
		apn.init(clonedNet);
//		context.getProvidedObjectManager().createProvidedObject("Accepting Petri net " + net.getLabel(), apn, AcceptingPetriNet.class,
//				context);

		/*
		 * Check whether the initial marking is known.
		 */
		try {
			Collection<InitialMarkingConnection> connections = context.getConnectionManager().getConnections(InitialMarkingConnection.class, context, net);
			if (connections.size() == 1) {
				/*
				 * Found unique initial marking, use it.
				 */
				apn.setInitialMarking(PetriNetUtils.apply((Marking)connections.iterator().next().getObjectWithRole(InitialMarkingConnection.MARKING), placeMap));
			}
		} catch (ConnectionCannotBeObtained e) {
		}
		
		/*
		 * Check existing final markings.
		 */
		try {
			Collection<FinalMarkingConnection> connections = context.getConnectionManager().getConnections(FinalMarkingConnection.class, context, net);
			Set<Marking> finalMarkings = new HashSet<Marking>();
			for (FinalMarkingConnection connection : connections) {
				finalMarkings.add(PetriNetUtils.apply((Marking)connection.getObjectWithRole(FinalMarkingConnection.MARKING), placeMap));
			}
			apn.setFinalMarkings(finalMarkings);
		} catch (ConnectionCannotBeObtained e) {
		}
		return apn;
	}
}
