package org.processmining.plugins;

import java.util.Collection;

import org.processmining.connections.ReduceUsingMurataRulesConnection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.AcceptingPetriNet;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.impl.DivideAndConquerFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.parameters.ReduceUsingMurataRulesParameters;
import org.processmining.plugins.petrinet.reduction.Murata;
import org.processmining.plugins.petrinet.reduction.MurataInput;
import org.processmining.plugins.petrinet.reduction.MurataOutput;
import org.processmining.utils.PetriNetUtils;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.acceptingpetrinetclassicalreductor.plugins.ReduceUsingMurataRulesPlugin instead.
 */
@Deprecated
//Plugin(name = "Reduce Using Murata Rules", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class })
public class ReduceUsingMurataRulesPlugin {

//	UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	PluginVariant(variantLabel = "Reduce Using Murata Rules, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		ReduceUsingMurataRulesParameters parameters = new ReduceUsingMurataRulesParameters();
		return reducePrivateConnection(context, net, parameters);
	}

//	PluginVariant(variantLabel = "Reduce Using Murata Rules, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		return reducePrivateConnection(context, net, parameters);
	}

	private AcceptingPetriNet reducePrivateConnection(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		Collection<ReduceUsingMurataRulesConnection> connections;
		try {
			connections = context.getConnectionManager().getConnections(ReduceUsingMurataRulesConnection.class,
					context, net);
			for (ReduceUsingMurataRulesConnection connection : connections) {
				if (connection.getObjectWithRole(ReduceUsingMurataRulesConnection.NET).equals(net)
						&& connection.getParameters().equals(parameters)) {
					return connection.getObjectWithRole(ReduceUsingMurataRulesConnection.REDUCEDNET);
				}
			}
		} catch (ConnectionCannotBeObtained e) {
		}
		AcceptingPetriNet reducedNet = reducePrivate(context, net, parameters);
		context.getProvidedObjectManager().createProvidedObject(
				"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(), Marking.class,
				context);
		context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
		context.getConnectionManager().addConnection(new ReduceUsingMurataRulesConnection(net, reducedNet, parameters));
		return reducedNet;
	}

	private AcceptingPetriNet reducePrivate(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		AcceptingPetriNet reducedNet = DivideAndConquerFactory.createAcceptingPetriNet();

		Murata murata = new Murata();
		MurataInput input = new MurataInput(net.getNet(), net.getInitialMarking());
		// Make all visible transitions sacred.
		input.setVisibleSacred(net.getNet());
		// Make all places involved in the initial marking scared.
		for (Place place : net.getInitialMarking().baseSet()) {
			input.addSacred(place);
		}
		// Make all places involved in a final marking scared.
		for (Marking finalMarking : net.getFinalMarkings()) {
			for (Place place : finalMarking.baseSet()) {
				input.addSacred(place);
			}
		}
		try {
			// Try to reduce non-sacred place and non-scared transitions.
			MurataOutput output = murata.run(null, input);
			reducedNet.init(output.getNet());
			reducedNet.setInitialMarking(PetriNetUtils.apply(net.getInitialMarking(), output.getPlaceMapping()));
			reducedNet.setFinalMarkings(PetriNetUtils.apply(net.getFinalMarkings(), output.getPlaceMapping()));
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return reducedNet;
	}

}
