package org.processmining.plugins;

import java.awt.Color;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.ActivityClusterArray;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.AttributeMap.ArrowType;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphEdge;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.shapes.Octagon;
import org.processmining.models.shapes.RoundedRect;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.activityclusterarray.plugins.VisualizeActivityClusterArray instead.
 */
@Deprecated
public class VisualizeActivityClusterArray {

//	Plugin(name = "Visualize Activity Cluster Array (undirected)", returnLabels = { "Visualized Activity Cluster Array" }, returnTypes = { JComponent.class }, parameterLabels = { "Causal Activity Matrix" }, userAccessible = false)
//	Visualizer
//	PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(UIPluginContext context, ActivityClusterArray clusters) {
		VisualizedGraph view = new VisualizedGraph(context, clusters);
		return ProMJGraphVisualizer.instance().visualizeGraph(context, view);
	}

	private class VisualizedNode extends AbstractDirectedGraphNode {

		VisualizedGraph view;

		public VisualizedNode(VisualizedGraph view, XEventClass activity) {
			this.view = view;
			getAttributeMap().put(AttributeMap.SHAPE, new RoundedRect());
			getAttributeMap().put(AttributeMap.SQUAREBB, false);
			getAttributeMap().put(AttributeMap.RESIZABLE, false);
			getAttributeMap().put(AttributeMap.SIZE, new Dimension(100, 60));
			getAttributeMap().put(AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			getAttributeMap().put(AttributeMap.LABEL, activity.getId());
		}

		public VisualizedNode(VisualizedGraph view, Integer index) {
			this.view = view;
			getAttributeMap().put(AttributeMap.SHAPE, new Octagon(0.3));
			getAttributeMap().put(AttributeMap.SQUAREBB, false);
			getAttributeMap().put(AttributeMap.RESIZABLE, false);
			getAttributeMap().put(AttributeMap.SIZE, new Dimension(100, 100));
			getAttributeMap().put(AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			getAttributeMap().put(AttributeMap.LABEL, "Cluster " + index.toString());
		}

		public AbstractDirectedGraph<?, ?> getGraph() {
			return view;
		}

	}

	private class VisualizedEdge extends AbstractDirectedGraphEdge<VisualizedNode, VisualizedNode> {

		public VisualizedEdge(VisualizedNode source, VisualizedNode target, ArrowType type) {
			super(source, target);
			getAttributeMap().put(AttributeMap.EDGEEND, type);
		}

	}

	private class VisualizedGraph extends AbstractDirectedGraph<VisualizedNode, VisualizedEdge> {

		private ActivityClusterArray clusters;
		private Map<Object, VisualizedNode> nodeMap = null;
		private Map<Pair<Object, Object>, VisualizedEdge> edgeMap = null;

		public VisualizedGraph(PluginContext context, ActivityClusterArray clusters) {
			this.clusters = clusters;
		}

		public void init() {
			if (nodeMap == null && edgeMap == null) {
				nodeMap = new HashMap<Object, VisualizedNode>();
				edgeMap = new HashMap<Pair<Object, Object>, VisualizedEdge>();
				int size = clusters.getClusters().size();
				for (int i = 0; i < size; i++) {
					nodeMap.put(i, new VisualizedNode(this, i));
					Set<XEventClass> cluster = clusters.getCluster(i);
					for (XEventClass activity : cluster) {
						boolean isIn = clusters.getOutputs(cluster).contains(activity);
						boolean isOut = clusters.getInputs(cluster).contains(activity);
						if (!nodeMap.containsKey(activity)) {
							nodeMap.put(activity, new VisualizedNode(this, activity));
						}
						if (isIn) {
							edgeMap.put(new Pair<Object, Object>(i, activity), new VisualizedEdge(nodeMap.get(i),
									nodeMap.get(activity), ArrowType.ARROWTYPE_CLASSIC));
						}
						if (isOut) {
							edgeMap.put(new Pair<Object, Object>(activity, i), new VisualizedEdge(
									nodeMap.get(activity), nodeMap.get(i), ArrowType.ARROWTYPE_CLASSIC));
						}
						if (!isIn && !isOut) {
							edgeMap.put(new Pair<Object, Object>(i, activity), new VisualizedEdge(nodeMap.get(i),
									nodeMap.get(activity), ArrowType.ARROWTYPE_NONE));
						}
					}
				}
			}
		}

		public Set<VisualizedNode> getNodes() {
			init();
			return new HashSet<VisualizedNode>(nodeMap.values());
		}

		public Set<VisualizedEdge> getEdges() {
			init();
			return new HashSet<VisualizedEdge>(edgeMap.values());
		}

		public void removeNode(DirectedGraphNode cell) {
		}

		protected AbstractDirectedGraph<VisualizedNode, VisualizedEdge> getEmptyClone() {
			return new VisualizedGraph(null, clusters);
		}

		protected Map<DirectedGraphElement, DirectedGraphElement> cloneFrom(
				DirectedGraph<VisualizedNode, VisualizedEdge> graph) {
			VisualizedGraph clone = new VisualizedGraph(null, this.clusters);
			clone.init();
			Map<DirectedGraphElement, DirectedGraphElement> map = new HashMap<DirectedGraphElement, DirectedGraphElement>();
			for (Object activity : nodeMap.keySet()) {
				map.put(this.nodeMap.get(activity), clone.nodeMap.get(activity));
			}
			return map;
		}

		public void removeEdge(DirectedGraphEdge edge) {
		}

	}
}
